#ifndef _ETHER_REG_H
#define _ETHER_REG_H

#include "xio.h"
#include "xbasic_types.h"

#ifndef BYTE
#define BYTE unsigned char
#endif
#ifndef WORD
#define WORD unsigned short
#endif

/* NE2000 definitions*/
#define NIC_BASE (0x00A00400) /* Base I/O address of the NIC card */
#define DATAPORT (0x10*2)
#define NE_RESET (0x1f*2)


/* NIC page0 register offsets */
#define CMDR    (0x00*2) /* command register for read & write*/
#define PSTART  (0x01*2) /* page start register for write*/
#define PSTOP   (0x02*2) /* page stop register for write*/
#define BNRY    (0x03*2) /* boundary reg for rd and wr*/
#define TPSR    (0x04*2) /* tx start page start reg for wr*/
#define TBCR0   (0x05*2) /* tx byte count 0 reg for wr*/
#define TBCR1   (0x06*2) /* tx byte count 1 reg for wr*/
#define ISR     (0x07*2) /* interrupt status reg for rd and wr*/
#define RSAR0   (0x08*2) /* low byte of remote start addr*/
#define RSAR1   (0x09*2) /* hi byte of remote start addr*/
#define RBCR0   (0x0A*2) /* remote byte count reg 0 for wr*/
#define RBCR1   (0x0B*2) /* remote byte count reg 1 for wr*/
#define RCR     (0x0C*2) /* rx configuration reg for wr*/
#define TCR     (0x0D*2) /* tx configuration reg for wr*/
#define DCR     (0x0E*2) /* data configuration reg for wr*/
#define IMR     (0x0F*2) /* interrupt mask reg for wr*/

/* NIC page 1 register offsets*/
#define PAR0    (0x01*2) /* physical addr reg 0 for rd and wr*/
#define CURR    (0x07*2) /* current page reg for rd and wr*/
#define MAR0    (0x08*2) /* multicast addr reg 0 for rd and WR*/

/* Buffer Length and Field Definition Info*/
#define TXSTART  0x40              /* Tx buffer start page*/
#define TXPAGES  6                 /* Pages for Tx buffer*/
#define RXSTART  (TXSTART+TXPAGES) /* Rx buffer start page*/
#define RXSTOP   0x7e              /* Rx buffer end page for word mode*/
#define DCRVAL   0x49              /* DCR values for word mode*/

/* macros for reading and writting registers*/
#define outnic(addr, data) XIo_Out16(NIC_BASE+addr, data)
#define innic(addr) XIo_In16(NIC_BASE+addr)


/*Structure Headers for Ethernet*/
#define MACLEN 6
typedef struct                  /* Net driver configuration data */
{
    WORD dtype;                     /* Driver type */
    BYTE myeth[MACLEN];             /* MAC (Ethernet) addr */
    WORD ebase;                     /* Card I/O base addr */
    WORD next_pkt;                  /* Next (current) Rx page */
} CONFIGNE;

typedef struct {                /* NIC hardware packet header */
    BYTE stat;                  /*     Error status */
    BYTE next;                  /*     Pointer to next block */
    WORD len;                   /*     Length of this frame incl. CRC */
} NICHDR;



/* Ether Function Prototypes*/
   void diag(); /* diagnostic to check if the ethernet is working*/
void delay(int mult); /*delay loop*/
int init_etherne(CONFIGNE* cp);
void resetnic(CONFIGNE* cp);
void dmaRead(void *data, WORD addr, WORD length);
void dmaWrite(WORD *data, WORD addr, WORD length);
int send(WORD len, WORD *packet);
void getnic(WORD addr, BYTE data[], WORD len);
void receive(int* len, BYTE* buffer);


/* Mac functions for the xilinx xilnet library 2.0.0 */
void XEmac_SendFrame(unsigned int MYMAC_BASEADDR,BYTE *pkt, int len);
/* return size integer*/
int XEmac_RecvFrame(unsigned int MYMAC_BASEADDR, BYTE *buf);


/*Global Variables*/
#define WORDMODE 1

static int promisc=0;
/* packet prototype */


#endif
